/* This DUT is a memory that also records how many */
/* reads and writes have been performed            */

module dut ( mem_interface mem_interface_h );
import mem_agent_pkg::*;

   logic [15:0]       instruction_count;
   logic [15:0]       memory [0:255];
   logic              test;

   always @( posedge mem_interface_h.clock
          or negedge mem_interface_h.reset ) begin

      // reset the outputs from this block
      if ( mem_interface_h.reset == 1'B1) begin
         mem_interface_h.data_from_dut       <= '0;
         mem_interface_h.data_from_dut_valid <=  0;
         mem_interface_h.dut_busy            <=  0;
         instruction_count                           <= '0;

      end else begin

         // wait until we are in a valid bus cycle
         if (mem_interface_h.data_to_dut_valid == 1'B1) begin
            //$display("DUT - in a bus cycle");

            test <= (mem_interface_h.data_to_dut_valid & mem_interface_h.dut_busy) ||
                    (~mem_interface_h.data_to_dut_valid & mem_interface_h.dut_busy) ||
                    (mem_interface_h.data_to_dut_valid & ~mem_interface_h.dut_busy);

            // perform the only function that makes this more complex than a
            // simple RAM :: pass out the number of reads / writes if the address is zero
            if (mem_interface_h.address == 16'H0000) begin
               //$display("DUT - writing out the number of instructions processed");
               mem_interface_h.data_from_dut <= instruction_count;

            // else manage the reads and writes to the RAM
            end else begin

               if (mem_interface_h.instruction == READ  ) begin
                  mem_interface_h.data_from_dut    <= memory [mem_interface_h.address];
                  //$display("DUT - READ cycle, writing out %d from address: ", memory [mem_interface_h.address],mem_interface_h.address );
                  instruction_count++;
               end

               if (mem_interface_h.instruction == WRITE ) begin
                  //$display("DUT - WRITE cycle, writing %d to memory address %d",mem_interface_h.data_to_dut, mem_interface_h.address);
                  memory[mem_interface_h.address] <= mem_interface_h.data_to_dut;
                  instruction_count++;
               end
            end

            // delay for the expected latency
            mem_interface_h.dut_busy <= 1;
            for (int i = 0; i < mem_interface_h.latency & i < 8;i++) begin
               @(posedge mem_interface_h.clock);
            end
            mem_interface_h.dut_busy <= 0;
         end
      end // else: !if( mem_interface_h.reset == 1'B1)
   end // always @ ( posedge mem_interface_h.clock...
endmodule

