////////////////////////////////////////////////////////////////////////////
// mem_agent_c - A unified sequence stimulus group using TLM
////////////////////////////////////////////////////////////////////////////
class mem_agent_c extends ovm_agent;

   const string report_id = "MEM_AGENT";

   mem_driver_c                              m_driver_h;
   ovm_sequencer #(mem_item_c, mem_item_c)   m_sequencer_h;
   mem_monitor_c                             m_monitor_h;
   ovm_analysis_port #(mem_item_c)           m_mon_out_ap;
   ovm_analysis_port #(mem_item_c)           m_drv_out_ap;

   mem_config_c                              m_mem_cfg_h;  //Config object handle

   //Register with the Factory
   `ovm_component_utils(mem_agent_c)

     function new(string name, ovm_component parent);
        super.new(name, parent);
     endfunction // new

   function void build();
      super.build();
      
      //Use the runtime verbosity package
      runtime_verbosity_pkg::set_verbosity(.comp(this), .report_id(report_id));
      
      //Get the config object
      m_mem_cfg_h = mem_config_c::get_config(this, "mem_config");

      //Build the driver and controller
      if (m_mem_cfg_h.has_driver) begin
         `ovm_info(report_id, "Getting us_driver_c from Factory", OVM_DEBUG )
         m_driver_h    = mem_driver_c::type_id::create("m_driver_h", this);
         // new the analysis port
         m_drv_out_ap = new("m_drv_out_ap", this);
      end

      if (m_mem_cfg_h.has_sequencer) begin
         `ovm_info(report_id, "Newing unified sequencer", OVM_DEBUG )
         m_sequencer_h = new("m_sequencer_h", this);
      end

      if (m_mem_cfg_h.has_monitor) begin
         `ovm_info(report_id, "Getting Monitor from the Factory", OVM_DEBUG )
         m_monitor_h   = mem_monitor_c::type_id::create("m_monitor_h", this);
         // new the analysis port
         m_mon_out_ap = new("m_mon_out_ap", this);
      end

      `ovm_info(report_id, get_full_name(), OVM_DEBUG)
      
      //Use the runtime verbosity package (called a second time to handle
      // recursive calls)
      runtime_verbosity_pkg::set_verbosity(.comp(this), .report_id(report_id));
   endfunction // build

   function void connect();
      super.connect();
      // connect up the ovm ports
      if (m_mem_cfg_h.has_driver && m_mem_cfg_h.has_sequencer) begin
         //Connect the driver's port
         m_driver_h.seq_item_port.connect(m_sequencer_h.seq_item_export);
      end
      if (m_mem_cfg_h.has_monitor) begin
         // connect the analysis port in the monitor to the agent port
         m_monitor_h.result_from_monitor_ap.connect(m_mon_out_ap);
      end
      if (m_mem_cfg_h.has_driver) begin
         // connect the analysis port in the driver to the agent port
         m_driver_h.sb_out_ap.connect(m_drv_out_ap);
      end
   endfunction // connect

endclass // mem_agent_c



   
