////////////////////////////////////////////////////////////////////////////
// mem_config_c - The config class for the mem_agent
////////////////////////////////////////////////////////////////////////////
class mem_config_c extends ovm_object;
   //Register with the factory
   `ovm_object_utils(mem_config_c);

   //Virtual Interface Handle
   virtual mem_interface             mem_interface;
   virtual mem_interface.monitor_mp  mem_intf_mon_mp;

   //Specify the config values
   bit      has_monitor   = 1;
   bit      has_driver    = 1;
   bit      has_sequencer = 1;
   
   string   testname;

   bit      enable_monitor_recording = 0; //Disable by default

   // Convenience function that first gets the object out of the OVM database
   // and reports an error if the object is not present in the database, then
   // casts it to the correct config object type, again checking for errors
   static function mem_config_c get_config( ovm_component comp, string cfg_name = "mem_cfg");
      string report_id = "MEM_CONFIG";
      ovm_object obj;
      mem_config_c mem_cfg_h;
      
      if( !comp.get_config_object( cfg_name , obj, .clone(0) ) ) begin
         comp.ovm_report_error( report_id,
                                $sformatf("%s has no config associated with id %s", comp.get_full_name(), cfg_name),,`__FILE__, `__LINE__ );
         return null;
      end
      
      if( !$cast( mem_cfg_h , obj ) ) begin 
         comp.ovm_report_error( report_id,
                                $sformatf("the object from the config space associated with id %s is of type $s which is not the required type %s" ,
                                       cfg_name , obj.get_type_name() , type_name ),,`__FILE__, `__LINE__ );
         return null;
      end
      return mem_cfg_h;
   endfunction : get_config
endclass
