`include "ovm_macros.svh"

/* a description of the wires used to connect the DUT to the testbench */
interface mem_interface();
   import ovm_pkg::*;
   import mem_agent_pkg::*;

   logic        clock;
   logic        reset;
   logic [ 7:0] address;
   logic [15:0] data_to_dut;
   logic [15:0] data_from_dut;   
   logic [ 7:0] latency;
   logic        data_from_dut_valid;           
   logic        data_to_dut_valid;           
   logic        dut_busy;           

   instruction_type instruction;

   //ModPort for the Monitor
   modport monitor_mp
    (
      output clock, reset, address, data_to_dut, data_from_dut, latency,
             data_from_dut_valid, data_to_dut_valid, dut_busy, instruction
    );
    
   // a simple assertion to check that the basic bus protocol is being followed
   property cycle_completed;
      @(posedge clock) disable iff (reset)
        ($rose(data_to_dut_valid) |=> dut_busy ##[1:11] !dut_busy && !data_to_dut_valid);
   endproperty
   cycle_completed_assertion : assert property(cycle_completed) 
     else `ovm_error("CYCLE COMPLETED ASSERTION", "Interface protocol not implemented correctly" )

   // generate a warning if latency's greater than 7 cycles are used as the DUT will truncate them.
   // in a real testbench this could be sent out of an analysis port as this may be valid but this level doesn't know that
   property check_latency_less_than_8;
      @(posedge clock) disable iff (reset)
        ($rose(data_to_dut_valid) |=> latency < 8);
   endproperty
   check_latency_less_than_8_assertion : assert property(check_latency_less_than_8) else `ovm_warning("CLOCK LATENCY ASSERTION", $sformatf("latency %d should be truncated by the DUT as is longer than 7 cycles!", latency) )

   // generate an error if latency's equal to 0 as the DUT will truncate never respond.
   // in a real testbench this could be sent out of an analysis port as this may be valid but this level doesn't know that
   property check_latency_equal_0;
      @(posedge clock) disable iff (reset)
        ($rose(data_to_dut_valid) |=> latency > 0);
   endproperty
   check_latency_equal_0_assertion : assert property(check_latency_equal_0) else `ovm_error("CLOCK LATENCY ASSERTION","latency is equal to 0.  The DUT will never respond!" )

   // Check for address 0 reads
   property check_address_0_read;
      @(posedge clock) disable iff (reset)
        ($rose(data_to_dut_valid) |-> address == 0 && instruction == READ);
   endproperty
   cover_address_0_read_assertion : cover property(check_address_0_read);

   // Check for address 0 writes
   property check_address_0_write;
      @(posedge clock) disable iff (reset)
        ($rose(data_to_dut_valid) |-> address == 0 && instruction == WRITE);
   endproperty
   cover_address_0_write_assertion : cover property(check_address_0_write);
         
endinterface : mem_interface
