//////////////////////////////////////////////////////////////////////////////
// mem_monitor_c - a monitor to connect the interface to the analysis group
//////////////////////////////////////////////////////////////////////////////
class mem_monitor_c extends ovm_monitor;

   const string report_id = "MEM_MONITOR";

   ovm_analysis_port #(mem_item_c)      result_from_monitor_ap;

   mem_item_c                           transaction_h;

   mem_config_c                         mem_cfg_h;  //Config object handle
   
   string                               testname;

   virtual mem_interface.monitor_mp     mem_interface_h;

   `ovm_component_utils(mem_monitor_c)

   function new(string name, ovm_component p);
      string message;
      super.new(name, p);
      `ovm_info(report_id, $sformatf("just built a monitor called %s", name), OVM_DEBUG)
   endfunction : new

   function void build();
      super.build();
      
      //Use the runtime verbosity package
      runtime_verbosity_pkg::set_verbosity(.comp(this), .report_id(report_id));
      
      //Get the config object
      mem_cfg_h = mem_config_c::get_config(this, "mem_config");
      
      // and add an analysis port for the read packets
      result_from_monitor_ap = new("result_from_monitor_ap", this);
   endfunction : build
   
   function void connect();
      mem_interface_h = mem_cfg_h.mem_intf_mon_mp;
   endfunction : connect

   function void configure();
      if (ovm_report_enabled(OVM_DEBUG ,OVM_INFO,report_id))
        print_config_settings();

      //Print the testname
      `ovm_info(report_id, mem_cfg_h.testname, OVM_HIGH )
   endfunction : configure


   task run();
      string message;
      `ovm_info(report_id,  "running monitor", OVM_HIGH )

      // on reset .....
      @(negedge mem_interface_h.reset);
      forever @(negedge mem_interface_h.dut_busy) begin

         transaction_h = new();

         // Note: using blocking assignments here becuase this is assigning to signals 
         //       declared in dynamic code.
         transaction_h.address       =  mem_interface_h.address       ;    
         transaction_h.data_to_dut   =  mem_interface_h.data_to_dut   ;    
         transaction_h.latency       =  mem_interface_h.latency       ;    
         transaction_h.instruction   =  mem_interface_h.instruction   ;    
         transaction_h.data_from_dut =  mem_interface_h.data_from_dut ;

         result_from_monitor_ap.write(transaction_h);

         $sformat(message,"sending an instruction to the scoreboard (address %d, instruction %s)", transaction_h.address, transaction_h.instruction);                     
         `ovm_info(report_id, message, OVM_HIGH )
      end
   endtask : run
endclass : mem_monitor_c
