//////////////////////////////////////////////////////////////////////////////
// coverage_c - the coverage collector.
// Record the write transations in an array.  When a read happens, compare
// the transaction with the previously recoreded write transations
//////////////////////////////////////////////////////////////////////////////
class coverage_c extends ovm_subscriber #(mem_item_c);

   string report_id = "COVERAGE";

   mem_item_c                           instruction_h;

   //Register with the Factory
   `ovm_component_utils(coverage_c)

   covergroup instruction_type_covergroup;
      //I want to see a total coverage number.  The default changed in the
      // 2009 LRM.
      type_option.merge_instances = 1;

      instruction_type: coverpoint instruction_h.instruction
        {
         bins         read    = { READ  };
         bins         write   = { WRITE };
         bins         bucket  = default;
         }
      latency:          coverpoint instruction_h.latency
        {
         bins         short   =  { [1:2] };
         bins         long    =  { [3:6] };
         bins         longer  =  {    7  };
         bins         longest =  {    8  };
         illegal_bins illegal = default;
         }
      latency_and_instruction: cross instruction_type, latency;
   endgroup : instruction_type_covergroup

   function new(string nm, ovm_component p);
      super.new(nm, p);
      instruction_type_covergroup = new;
   endfunction : new

   function void build();
      super.build();

      //Void casting this call, since it's fine to use the
      // default if not overridden
      void'(get_config_string("report_id", report_id));

      //Use the runtime verbosity package
      runtime_verbosity_pkg::set_verbosity(.comp(this), .report_id(report_id));
   endfunction : build

   //The write function is comes from the ovm_analysis_imp (implementation) that comes from the
   // parent class ovm_subscriber.  When someone that is connected to the analysis_export that
   // ovm_subscriber provides calls write, this function is ultimately called.
   function void write(T t);
      instruction_h = t;
      instruction_type_covergroup.sample();
   endfunction : write

endclass : coverage_c
