//////////////////////////////////////////////////////////////////////////////
// environment_c - .the environment holding all the objects for the TB
//////////////////////////////////////////////////////////////////////////////
class environment_c extends ovm_env;

   string report_id = "ENVIRONMENT";

   mem_agent_c                                 m_mem_agent_h;
   analysis_group_c                            m_analysis_h;

   `ovm_component_utils(environment_c)

   function new(string name, ovm_component parent = null);
      super.new(name, parent);
   endfunction : new

   function void build();
      super.build();

      //Void casting this call, since it's fine to use the
      // default if not overridden
      void'(get_config_string("report_id", report_id));

      //Use the runtime verbosity package
      runtime_verbosity_pkg::set_verbosity(.comp(this), .report_id(report_id));

      //Create the components in the environment using the Factory
      `ovm_info(report_id,"Getting Mem Agent from the Factory", OVM_DEBUG)
      m_mem_agent_h = mem_agent_c::type_id::create("m_mem_agent_h",this);

      `ovm_info(report_id,"Getting Analysis Group from the Factory", OVM_DEBUG)
      m_analysis_h = analysis_group_c::type_id::create("m_analysis_h", this);

      //Use the runtime verbosity package - second call to support recursion
      runtime_verbosity_pkg::set_verbosity(.comp(this), .report_id(report_id));
   endfunction : build

   function void connect();
      `ovm_info(report_id, "connecting analysis port", OVM_DEBUG)
      // connect up the ovm ports
      // connect the analysis port in the monitor to the scoreboard get port
      m_mem_agent_h.m_mon_out_ap.connect(m_analysis_h.analysis_export);
   endfunction : connect

   function void pre_run();
      super.pre_run();
      //factory.print();
   endfunction : pre_run

   function void report();
      super.report();
      `ovm_info(report_id, "Finished Test",OVM_LOW )
      //factory.print();
   endfunction : report

endclass : environment_c
