//////////////////////////////////////////////////////////////////////////////
// scoreboard_c - The scoreboard for comparisons.
// Record the write transations in an array.  When a read happens, compare
// the transaction with the previously recoreded write transations
//////////////////////////////////////////////////////////////////////////////
class scoreboard_c extends ovm_component;

   string report_id = "SCOREBOARD";

   ovm_analysis_export   #(mem_item_c)  analysis_export;
   local analysis_fifo   #(mem_item_c)  afifo_input_from_monitor;

   mem_item_c                   instruction_h;
   mem_item_c                   addresses_written_list [int];
   local      int               instructions_sent;

   //Register with the Factory
   `ovm_component_utils(scoreboard_c)

   function new(string nm, ovm_component p);
      super.new(nm, p);
      instructions_sent = 0;
   endfunction : new

   function void build();
      super.build();

      //Void casting this call, since it's fine to use the
      // default if not overridden
      void'(get_config_string("report_id", report_id));

      //Use the runtime verbosity package
      runtime_verbosity_pkg::set_verbosity(.comp(this), .report_id(report_id));

      //New the fifo and export
      `ovm_info(report_id, "Building analysis fifo and export", OVM_DEBUG)
      afifo_input_from_monitor  = new("afifo_input_from_monitor",  this);
      analysis_export           = new("analysis_export",           this);

      //Use the runtime verbosity package - second call to support recursion
      runtime_verbosity_pkg::set_verbosity(.comp(this), .report_id(report_id));
   endfunction : build

   function void connect();
      //Connect the fifo and export up
      analysis_export.connect(afifo_input_from_monitor.analysis_export);
   endfunction : connect

   task run();
      // take a copy of write transactions so they can be compared when a read happens
      forever begin
         string message;
         afifo_input_from_monitor.get(instruction_h);

         if (instruction_h.instruction == WRITE) begin
            instructions_sent++;
            $sformat(message, "Saving instruction address %d during write", instruction_h.address);
            `ovm_info(report_id, message, OVM_HIGH )
            addresses_written_list[instruction_h.address] = new();
            addresses_written_list[instruction_h.address].copy(instruction_h);
         end

         if (instruction_h.instruction == READ) begin
            if (instruction_h.address == 0) begin
               $sformat(message, "Read address 0, number of instructions = %d, dut reports: %d", instructions_sent, instruction_h.data_from_dut);
               if (instructions_sent == instruction_h.data_from_dut) begin
                  `ovm_info(report_id, message, OVM_HIGH )
               end else begin
                  `ovm_warning(report_id, message)
               end
            end else begin
               instructions_sent++;
               $sformat(message, "Read address %d, ", instruction_h.address);
               `ovm_info(report_id, message, OVM_HIGH )
               if (!(instruction_h.compare(addresses_written_list[instruction_h.address]))) begin
                  `ovm_error(report_id, "instruction missmatch!!" )
               end
            end
         end
      end
   endtask : run
endclass : scoreboard_c
