`include "ovm_macros.svh"

/* this file contains the clock and reset module    */
/* and the top level of the testbench               */
module clock_reset(interface i );
   parameter bit ACTIVE_RESET = 1;
   import ovm_pkg::*;

   process clk_proc;

   task run(int reset_hold = 4,
            int half_period = 10,
            int count = 0 );
      clk_proc = process::self();

      i.clock = 0;
      i.reset = ACTIVE_RESET;

      `ovm_info("CLOCKING", "running clock reset",OVM_LOW )

      for(int reset_i = 0;reset_i < reset_hold; reset_i++ ) begin
         #half_period; i.clock = !i.clock;
         #half_period; i.clock = !i.clock;
      end

      i.reset <= !i.reset;
      // Run the clock
      for(int clock_i = 0;(clock_i < count || count == 0);clock_i++ ) begin
         #half_period; i.clock = !i.clock;
      end
   endtask

   task kill();
     @(posedge i.clock);
     clk_proc.kill();
   endtask

endmodule

module testbench;
   import ovm_pkg::*;
   import ovm_tests_pkg::*;
   import ovm_container_pkg::*;

   mem_interface                mem_interface_i();

   clock_reset                   clock_reset_inst  ( mem_interface_i );
   dut                           dut_inst          ( mem_interface_i );

   //vif_container #(virtual mem_interface,
   //                virtual mem_interface.monitor_mp) vif_container_h = new();

   initial begin
      //Kick off the clock
      fork
         clock_reset_inst.run();
      join_none

      //TimeFormat to make the time displays pretty and to allow you to double
      // click on the time in the transcript window and then have the wave
      // cursor jump to that time.
      $timeformat(-9, 0, " ns", 5);

      //Use the ovm_container to put the mem_interface into the config space
      ovm_container #(virtual mem_interface)::
        set_value_in_global_config("mem_interface",
                                   mem_interface_i);

      //Use the ovm_container to put the mem_interface monitor modport in the
      // config space
      ovm_container #(virtual mem_interface.monitor_mp)::
        set_value_in_global_config("mem_intf_mon_mp",
                                   mem_interface_i.monitor_mp);

      //Turn on extra debug messages
      //ovm_top.set_report_verbosity_level_hier(12000);

      //Run the test (OVM_TESTNAME will override "test1" if set on cmd line)
      run_test("test1");
      clock_reset_inst.kill();
   end

endmodule // testbench






