//////////////////////////////////////////////////////////////////////////////
// test_base_c - Base class for all tests.
//////////////////////////////////////////////////////////////////////////////
virtual class test_base_c extends ovm_test;

   const string report_id = "TEST_BASE";

   //Config object handles
   mem_config_c                                m_mem_cfg_h;
   
   //Environment and unified sequence handles
   environment_c                               m_env_h;
   mem_seq_base_c                              m_mem_seq_h;
   
   //This testbench only uses one controller.  We could have several sequencers
   // and then control ordering in the testbench on different  interfaces 
   // right here in the test by starting sequences in a specific order.
   ovm_sequencer #(mem_item_c, mem_item_c)     m_mem_sequencer_h;

   `ovm_component_utils(test_base_c)

   function new(string name = "test_base_c", ovm_component parent=null);
      super.new(name, parent);
   endfunction // new

   function void build();
      super.build();
      
      //Turn off the default simple sequence
      set_config_int("*", "count", 0);
      
      //Get the environment
      m_env_h = environment_c::type_id::create("m_env_h", this);
      //Create the mem sequence (This will be overriden in the test
      //  extensions)
      m_mem_seq_h = mem_seq_base_c::type_id::create("m_mem_seq_h", this);
      
   endfunction : build
  
  function void end_of_elaboration();
     ovm_component tmp[$];
     super.end_of_elaboration();
     //find the mem sequencer in the testbench
     tmp.delete(); //Make sure the queue is empty
     ovm_top.find_all("*mem_agent_h.m_sequencer_h", tmp);
     if (tmp.size() == 0)
       `ovm_fatal(report_id, "Failed to find mem sequencer")
     else if (tmp.size() > 1)
       `ovm_fatal(report_id, "Matched too many components when looking for mem sequencer")  
     else
       $cast(m_mem_sequencer_h, tmp[0]);
  endfunction : end_of_elaboration

   virtual task run();
      `ovm_fatal(report_id, "We didn't override the run() task." )
   endtask : run

   //Used to set default configuration for the mem_agent
   virtual function void configure_mem(mem_config_c mem_cfg_h,
                                       string testname);
      //Set the config values
      mem_cfg_h.testname = testname;
      //Turn on transaction recording in the monitor
      mem_cfg_h.enable_monitor_recording = 0;
      //Set the vif handles
      mem_cfg_h.mem_interface   = ovm_container #(virtual mem_interface)::
                                      get_value_from_config(this,
                                                            "mem_interface");
      mem_cfg_h.mem_intf_mon_mp = ovm_container #(virtual mem_interface.monitor_mp)::
                                      get_value_from_config(this,
                                                            "mem_intf_mon_mp");
   endfunction : configure_mem
   
endclass : test_base_c
